@@ -19,8 +19,12 @@
  */
 
 #ifdef _WIN32
+#ifdef __MINGW64__
+#define _WIN32_IE 0x501
+#else
 #define _WIN32_IE 0x400
 #endif
+#endif
 
 #include "config.h"
 
@@ -211,7 +215,7 @@ void ReadALConfig(void)
 #ifdef _WIN32
     if(SHGetSpecialFolderPathA(NULL, buffer, CSIDL_APPDATA, FALSE) != FALSE)
     {
-        int p = strlen(buffer);
+        size_t p = strlen(buffer);
         snprintf(buffer+p, sizeof(buffer)-p, "\\alsoft.ini");
         f = fopen(buffer, "rt");
         if(f)
