@@ -1,33 +1,30 @@
 #ifndef _AL_SOURCE_H_
 #define _AL_SOURCE_H_
 
-#define AL_NUM_SOURCE_PARAMS    128
-
-/* This cannot be changed without working on the code! */
-#define MAX_SENDS                 1
+#define MAX_SENDS                 2
 
 #include "alFilter.h"
 #include "alu.h"
 #include "AL/al.h"
 
-#define AL_DIRECT_FILTER                                   0x20005
-#define AL_AUXILIARY_SEND_FILTER                           0x20006
-#define AL_AIR_ABSORPTION_FACTOR                           0x20007
-#define AL_ROOM_ROLLOFF_FACTOR                             0x20008
-#define AL_CONE_OUTER_GAINHF                               0x20009
-#define AL_DIRECT_FILTER_GAINHF_AUTO                       0x2000A
-#define AL_AUXILIARY_SEND_FILTER_GAIN_AUTO                 0x2000B
-#define AL_AUXILIARY_SEND_FILTER_GAINHF_AUTO               0x2000C
-
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+typedef enum {
+    POINT_RESAMPLER = 0,
+    LINEAR_RESAMPLER,
+    COSINE_RESAMPLER,
+
+    RESAMPLER_MAX,
+    RESAMPLER_MIN = -1,
+    RESAMPLER_DEFAULT = LINEAR_RESAMPLER
+} resampler_t;
+extern resampler_t DefaultResampler;
+
 typedef struct ALbufferlistitem
 {
-    ALuint                   buffer;
-    ALuint                   bufferstate;
-    ALuint                   flag;
+    struct ALbuffer         *buffer;
     struct ALbufferlistitem *next;
 } ALbufferlistitem;
 
@@ -50,26 +47,23 @@ typedef struct ALsource
     ALboolean    bLooping;
     ALenum       DistanceModel;
 
-    ALuint       ulBufferID;
+    resampler_t  Resampler;
 
-    ALboolean    inuse;
-    ALboolean    play;
     ALenum       state;
     ALuint       position;
     ALuint       position_fraction;
 
+    struct ALbuffer *Buffer;
+
     struct ALbufferlistitem *queue; // Linked list of buffers in queue
     ALuint       BuffersInQueue;    // Number of buffers in queue
-
-    ALuint TotalBufferDataSize; // Total amount of data contained in the buffers queued for this source
-    ALuint BuffersPlayed;       // Number of buffers played on this loop
+    ALuint       BuffersPlayed;     // Number of buffers played on this loop
 
     ALfilter DirectFilter;
 
     struct {
         struct ALeffectslot *Slot;
         ALfilter WetFilter;
-        FILTER iirFilter;
     } Send[MAX_SENDS];
 
     ALboolean DryGainHFAuto;
@@ -77,17 +71,10 @@ typedef struct ALsource
     ALboolean WetGainHFAuto;
     ALfloat   OuterGainHF;
 
-    FILTER iirFilter;
-
     ALfloat AirAbsorptionFactor;
-
     ALfloat RoomRolloffFactor;
-
     ALfloat DopplerFactor;
 
-    // Index to itself
-    ALuint source;
-
     ALint  lOffset;
     ALint  lOffsetType;
 
@@ -96,9 +83,28 @@ typedef struct ALsource
 
     // Current gains, which are ramped while mixed
     ALfloat DryGains[OUTPUTCHANNELS];
-    ALfloat WetGain;
+    ALfloat WetGains[MAX_SENDS];
     ALboolean FirstStart;
 
+    // Current target parameters used for mixing
+    ALboolean NeedsUpdate;
+    struct {
+        ALfloat DryGains[OUTPUTCHANNELS];
+        ALfloat WetGains[MAX_SENDS];
+        ALfloat Pitch;
+
+        struct {
+            FILTER iirFilter;
+            ALfloat history[OUTPUTCHANNELS];
+        } Send[MAX_SENDS];
+
+        FILTER iirFilter;
+        ALfloat history[OUTPUTCHANNELS*2];
+    } Params;
+
+    // Index to itself
+    ALuint source;
+
     struct ALsource *next;
 } ALsource;
 
