@@ -32,12 +32,14 @@
 #include "alThunk.h"
 
 
+DECL_VERIFIER(Databuffer, ALdatabuffer, databuffer)
+
 /*
 *    alGenDatabuffersEXT(ALsizei n, ALuint *puiBuffers)
 *
 *    Generates n AL Databuffers, and stores the Databuffers Names in the array pointed to by puiBuffers
 */
-ALvoid ALAPIENTRY alGenDatabuffersEXT(ALsizei n,ALuint *puiBuffers)
+AL_API ALvoid AL_APIENTRY alGenDatabuffersEXT(ALsizei n,ALuint *puiBuffers)
 {
     ALCcontext *Context;
     ALsizei i=0;
@@ -54,18 +56,28 @@ ALvoid ALAPIENTRY alGenDatabuffersEXT(ALsizei n,ALuint *puiBuffers)
          * Databuffer Names) */
         if(!IsBadWritePtr((void*)puiBuffers, n * sizeof(ALuint)))
         {
-            ALdatabuffer **list = &device->Databuffers;
+            ALdatabuffer *end;
+            ALdatabuffer **list = &device->DatabufferList;
             while(*list)
                 list = &(*list)->next;
 
             /* Create all the new Databuffers */
+            end = *list;
             while(i < n)
             {
                 *list = calloc(1, sizeof(ALdatabuffer));
                 if(!(*list))
                 {
-                    alDeleteDatabuffersEXT(i, puiBuffers);
-                    alSetError(AL_OUT_OF_MEMORY);
+                    while(end->next)
+                    {
+                        ALdatabuffer *temp = end->next;
+                        end->next = temp->next;
+
+                        ALTHUNK_REMOVEENTRY(temp->databuffer);
+                        device->DatabufferCount--;
+                        free(temp);
+                    }
+                    alSetError(Context, AL_OUT_OF_MEMORY);
                     break;
                 }
 
@@ -79,7 +91,7 @@ ALvoid ALAPIENTRY alGenDatabuffersEXT(ALsizei n,ALuint *puiBuffers)
             }
         }
         else
-            alSetError(AL_INVALID_VALUE);
+            alSetError(Context, AL_INVALID_VALUE);
     }
 
     ProcessContext(Context);
@@ -90,7 +102,7 @@ ALvoid ALAPIENTRY alGenDatabuffersEXT(ALsizei n,ALuint *puiBuffers)
 *
 *    Deletes the n AL Databuffers pointed to by puiBuffers
 */
-ALvoid ALAPIENTRY alDeleteDatabuffersEXT(ALsizei n, const ALuint *puiBuffers)
+AL_API ALvoid AL_APIENTRY alDeleteDatabuffersEXT(ALsizei n, const ALuint *puiBuffers)
 {
     ALCcontext *Context;
     ALdatabuffer *ALBuf;
@@ -109,26 +121,26 @@ ALvoid ALAPIENTRY alDeleteDatabuffersEXT(ALsizei n, const ALuint *puiBuffers)
          * deleted */
         for(i = 0;i < n;i++)
         {
-            /* Check for valid Buffer ID (can be NULL buffer) */
-            if(alIsDatabufferEXT(puiBuffers[i]))
+            if(!puiBuffers[i])
+                continue;
+
+            /* Check for valid Buffer ID */
+            if((ALBuf=VerifyDatabuffer(device->DatabufferList, puiBuffers[i])) != NULL)
             {
-                /* If not the NULL buffer, check that it's unmapped */
-                ALBuf = ((ALdatabuffer *)ALTHUNK_LOOKUPENTRY(puiBuffers[i]));
-                if(ALBuf)
+                if(ALBuf->state != UNMAPPED)
                 {
-                    if(ALBuf->state != UNMAPPED)
-                    {
-                        /* Databuffer still in use, cannot be deleted */
-                        alSetError(AL_INVALID_OPERATION);
-                        bFailed = AL_TRUE;
-                    }
+                    /* Databuffer still in use, cannot be deleted */
+                    alSetError(Context, AL_INVALID_OPERATION);
+                    bFailed = AL_TRUE;
+                    break;
                 }
             }
             else
             {
                 /* Invalid Databuffer */
-                alSetError(AL_INVALID_NAME);
+                alSetError(Context, AL_INVALID_NAME);
                 bFailed = AL_TRUE;
+                break;
             }
         }
 
@@ -138,11 +150,10 @@ ALvoid ALAPIENTRY alDeleteDatabuffersEXT(ALsizei n, const ALuint *puiBuffers)
         {
             for(i = 0;i < n;i++)
             {
-                if(puiBuffers[i] && alIsDatabufferEXT(puiBuffers[i]))
+                if((ALBuf=VerifyDatabuffer(device->DatabufferList, puiBuffers[i])) != NULL)
                 {
-                    ALdatabuffer **list = &device->Databuffers;
+                    ALdatabuffer **list = &device->DatabufferList;
 
-                    ALBuf = (ALdatabuffer*)ALTHUNK_LOOKUPENTRY(puiBuffers[i]);
                     while(*list && *list != ALBuf)
                         list = &(*list)->next;
 
@@ -167,7 +178,7 @@ ALvoid ALAPIENTRY alDeleteDatabuffersEXT(ALsizei n, const ALuint *puiBuffers)
         }
     }
     else
-        alSetError(AL_INVALID_VALUE);
+        alSetError(Context, AL_INVALID_VALUE);
 
     ProcessContext(Context);
 }
@@ -177,22 +188,23 @@ ALvoid ALAPIENTRY alDeleteDatabuffersEXT(ALsizei n, const ALuint *puiBuffers)
 *
 *    Checks if ulBuffer is a valid Databuffer Name
 */
-ALboolean ALAPIENTRY alIsDatabufferEXT(ALuint uiBuffer)
+AL_API ALboolean AL_APIENTRY alIsDatabufferEXT(ALuint uiBuffer)
 {
     ALCcontext *Context;
-    ALdatabuffer *ALBuf;
+    ALboolean  result = AL_TRUE;
+    ALCdevice *device;
 
     Context = GetContextSuspended();
     if(!Context) return AL_FALSE;
 
-    /* Check through list of generated databuffers for uiBuffer */
-    ALBuf = Context->Device->Databuffers;
-    while(ALBuf && ALBuf->databuffer != uiBuffer)
-        ALBuf = ALBuf->next;
+    device = Context->Device;
+    if(uiBuffer)
+        result = (VerifyDatabuffer(device->DatabufferList, uiBuffer) ?
+                  AL_TRUE : AL_FALSE);
 
     ProcessContext(Context);
 
-    return ((ALBuf || !uiBuffer) ? AL_TRUE : AL_FALSE);
+    return result;
 }
 
 /*
@@ -200,18 +212,19 @@ ALboolean ALAPIENTRY alIsDatabufferEXT(ALuint uiBuffer)
 *
 *    Fill databuffer with data
 */
-ALvoid ALAPIENTRY alDatabufferDataEXT(ALuint buffer,const ALvoid *data,ALsizei size,ALenum usage)
+AL_API ALvoid AL_APIENTRY alDatabufferDataEXT(ALuint buffer,const ALvoid *data,ALsizeiptrEXT size,ALenum usage)
 {
     ALCcontext *Context;
     ALdatabuffer *ALBuf;
+    ALCdevice *Device;
     ALvoid *temp;
 
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if(alIsDatabufferEXT(buffer) && buffer != 0)
+    Device = Context->Device;
+    if((ALBuf=VerifyDatabuffer(Device->DatabufferList, buffer)) != NULL)
     {
-        ALBuf = (ALdatabuffer*)ALTHUNK_LOOKUPENTRY(buffer);
         if(ALBuf->state == UNMAPPED)
         {
             if(usage == AL_STREAM_WRITE_EXT || usage == AL_STREAM_READ_EXT ||
@@ -220,284 +233,303 @@ ALvoid ALAPIENTRY alDatabufferDataEXT(ALuint buffer,const ALvoid *data,ALsizei s
                usage == AL_DYNAMIC_WRITE_EXT || usage == AL_DYNAMIC_READ_EXT ||
                usage == AL_DYNAMIC_COPY_EXT)
             {
-                /* (Re)allocate data */
-                temp = realloc(ALBuf->data, size);
-                if(temp)
+                if(size >= 0)
                 {
-                    ALBuf->data = temp;
-                    ALBuf->size = size;
-                    ALBuf->usage = usage;
-                    if(data)
-                        memcpy(ALBuf->data, data, size);
+                    /* (Re)allocate data */
+                    temp = realloc(ALBuf->data, size);
+                    if(temp)
+                    {
+                        ALBuf->data = temp;
+                        ALBuf->size = size;
+                        ALBuf->usage = usage;
+                        if(data)
+                            memcpy(ALBuf->data, data, size);
+                    }
+                    else
+                        alSetError(Context, AL_OUT_OF_MEMORY);
                 }
                 else
-                    alSetError(AL_OUT_OF_MEMORY);
+                    alSetError(Context, AL_INVALID_VALUE);
             }
             else
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
         }
         else
-            alSetError(AL_INVALID_OPERATION);
+            alSetError(Context, AL_INVALID_OPERATION);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(Context, AL_INVALID_NAME);
 
     ProcessContext(Context);
 }
 
-ALvoid ALAPIENTRY alDatabufferSubDataEXT(ALuint uiBuffer, ALuint start, ALsizei length, const ALvoid *data)
+AL_API ALvoid AL_APIENTRY alDatabufferSubDataEXT(ALuint uiBuffer, ALintptrEXT start, ALsizeiptrEXT length, const ALvoid *data)
 {
     ALCcontext    *pContext;
     ALdatabuffer  *pBuffer;
+    ALCdevice     *Device;
 
     pContext = GetContextSuspended();
     if(!pContext) return;
 
-    if(alIsDatabufferEXT(uiBuffer) && uiBuffer != 0)
+    Device = pContext->Device;
+    if((pBuffer=VerifyDatabuffer(Device->DatabufferList, uiBuffer)) != NULL)
     {
-        pBuffer = (ALdatabuffer*)ALTHUNK_LOOKUPENTRY(uiBuffer);
-
-        if(length >= 0 && start+length <= pBuffer->size)
+        if(start >= 0 && length >= 0 && start+length <= pBuffer->size)
         {
             if(pBuffer->state == UNMAPPED)
                 memcpy(pBuffer->data+start, data, length);
             else
-                alSetError(AL_INVALID_OPERATION);
+                alSetError(pContext, AL_INVALID_OPERATION);
         }
         else
-            alSetError(AL_INVALID_VALUE);
+            alSetError(pContext, AL_INVALID_VALUE);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(pContext, AL_INVALID_NAME);
 
     ProcessContext(pContext);
 }
 
-ALvoid ALAPIENTRY alGetDatabufferSubDataEXT(ALuint uiBuffer, ALuint start, ALsizei length, ALvoid *data)
+AL_API ALvoid AL_APIENTRY alGetDatabufferSubDataEXT(ALuint uiBuffer, ALintptrEXT start, ALsizeiptrEXT length, ALvoid *data)
 {
     ALCcontext    *pContext;
     ALdatabuffer  *pBuffer;
+    ALCdevice     *Device;
 
     pContext = GetContextSuspended();
     if(!pContext) return;
 
-    if(alIsDatabufferEXT(uiBuffer) && uiBuffer != 0)
+    Device = pContext->Device;
+    if((pBuffer=VerifyDatabuffer(Device->DatabufferList, uiBuffer)) != NULL)
     {
-        pBuffer = (ALdatabuffer*)ALTHUNK_LOOKUPENTRY(uiBuffer);
-
-        if(length >= 0 && start+length <= pBuffer->size)
+        if(start >= 0 && length >= 0 && start+length <= pBuffer->size)
         {
             if(pBuffer->state == UNMAPPED)
                 memcpy(data, pBuffer->data+start, length);
             else
-                alSetError(AL_INVALID_OPERATION);
+                alSetError(pContext, AL_INVALID_OPERATION);
         }
         else
-            alSetError(AL_INVALID_VALUE);
+            alSetError(pContext, AL_INVALID_VALUE);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(pContext, AL_INVALID_NAME);
 
     ProcessContext(pContext);
 }
 
 
-ALvoid ALAPIENTRY alDatabufferfEXT(ALuint buffer, ALenum eParam, ALfloat flValue)
+AL_API ALvoid AL_APIENTRY alDatabufferfEXT(ALuint buffer, ALenum eParam, ALfloat flValue)
 {
     ALCcontext    *pContext;
+    ALCdevice     *Device;
 
     (void)flValue;
 
     pContext = GetContextSuspended();
     if(!pContext) return;
 
-    if(alIsDatabufferEXT(buffer) && buffer != 0)
+    Device = pContext->Device;
+    if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
     {
         switch(eParam)
         {
         default:
-            alSetError(AL_INVALID_ENUM);
+            alSetError(pContext, AL_INVALID_ENUM);
             break;
         }
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(pContext, AL_INVALID_NAME);
 
     ProcessContext(pContext);
 }
 
-ALvoid ALAPIENTRY alDatabufferfvEXT(ALuint buffer, ALenum eParam, const ALfloat* flValues)
+AL_API ALvoid AL_APIENTRY alDatabufferfvEXT(ALuint buffer, ALenum eParam, const ALfloat* flValues)
 {
     ALCcontext    *pContext;
+    ALCdevice     *Device;
 
     (void)flValues;
 
     pContext = GetContextSuspended();
     if(!pContext) return;
 
-    if(alIsDatabufferEXT(buffer) && buffer != 0)
+    Device = pContext->Device;
+    if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
     {
         switch(eParam)
         {
         default:
-            alSetError(AL_INVALID_ENUM);
+            alSetError(pContext, AL_INVALID_ENUM);
             break;
         }
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(pContext, AL_INVALID_NAME);
 
     ProcessContext(pContext);
 }
 
 
-ALvoid ALAPIENTRY alDatabufferiEXT(ALuint buffer, ALenum eParam, ALint lValue)
+AL_API ALvoid AL_APIENTRY alDatabufferiEXT(ALuint buffer, ALenum eParam, ALint lValue)
 {
     ALCcontext    *pContext;
+    ALCdevice     *Device;
 
     (void)lValue;
 
     pContext = GetContextSuspended();
     if(!pContext) return;
 
-    if(alIsDatabufferEXT(buffer) && buffer != 0)
+    Device = pContext->Device;
+    if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
     {
         switch(eParam)
         {
         default:
-            alSetError(AL_INVALID_ENUM);
+            alSetError(pContext, AL_INVALID_ENUM);
             break;
         }
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(pContext, AL_INVALID_NAME);
 
     ProcessContext(pContext);
 }
 
-ALvoid ALAPIENTRY alDatabufferivEXT(ALuint buffer, ALenum eParam, const ALint* plValues)
+AL_API ALvoid AL_APIENTRY alDatabufferivEXT(ALuint buffer, ALenum eParam, const ALint* plValues)
 {
     ALCcontext    *pContext;
+    ALCdevice     *Device;
 
     (void)plValues;
 
     pContext = GetContextSuspended();
     if(!pContext) return;
 
-    if(alIsDatabufferEXT(buffer) && buffer != 0)
+    Device = pContext->Device;
+    if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
     {
         switch(eParam)
         {
         default:
-            alSetError(AL_INVALID_ENUM);
+            alSetError(pContext, AL_INVALID_ENUM);
             break;
         }
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(pContext, AL_INVALID_NAME);
 
     ProcessContext(pContext);
 }
 
 
-ALvoid ALAPIENTRY alGetDatabufferfEXT(ALuint buffer, ALenum eParam, ALfloat *pflValue)
+AL_API ALvoid AL_APIENTRY alGetDatabufferfEXT(ALuint buffer, ALenum eParam, ALfloat *pflValue)
 {
     ALCcontext    *pContext;
+    ALCdevice     *Device;
 
     pContext = GetContextSuspended();
     if(!pContext) return;
 
     if(pflValue)
     {
-        if(alIsDatabufferEXT(buffer) && buffer != 0)
+        Device = pContext->Device;
+        if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
         {
             switch(eParam)
             {
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(pContext, AL_INVALID_ENUM);
                 break;
             }
         }
         else
-            alSetError(AL_INVALID_NAME);
+            alSetError(pContext, AL_INVALID_NAME);
     }
     else
-        alSetError(AL_INVALID_VALUE);
+        alSetError(pContext, AL_INVALID_VALUE);
 
     ProcessContext(pContext);
 }
 
-ALvoid ALAPIENTRY alGetDatabufferfvEXT(ALuint buffer, ALenum eParam, ALfloat* pflValues)
+AL_API ALvoid AL_APIENTRY alGetDatabufferfvEXT(ALuint buffer, ALenum eParam, ALfloat* pflValues)
 {
     ALCcontext    *pContext;
+    ALCdevice     *Device;
 
     pContext = GetContextSuspended();
     if(!pContext) return;
 
     if(pflValues)
     {
-        if(alIsDatabufferEXT(buffer) && buffer != 0)
+        Device = pContext->Device;
+        if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
         {
             switch(eParam)
             {
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(pContext, AL_INVALID_ENUM);
                 break;
             }
         }
         else
-            alSetError(AL_INVALID_NAME);
+            alSetError(pContext, AL_INVALID_NAME);
     }
     else
-        alSetError(AL_INVALID_VALUE);
+        alSetError(pContext, AL_INVALID_VALUE);
 
     ProcessContext(pContext);
 }
 
-ALvoid ALAPIENTRY alGetDatabufferiEXT(ALuint buffer, ALenum eParam, ALint *plValue)
+AL_API ALvoid AL_APIENTRY alGetDatabufferiEXT(ALuint buffer, ALenum eParam, ALint *plValue)
 {
     ALCcontext    *pContext;
     ALdatabuffer  *pBuffer;
+    ALCdevice     *Device;
 
     pContext = GetContextSuspended();
     if(!pContext) return;
 
     if(plValue)
     {
-        if(alIsDatabufferEXT(buffer) && buffer != 0)
+        Device = pContext->Device;
+        if((pBuffer=VerifyDatabuffer(Device->DatabufferList, buffer)) != NULL)
         {
-            pBuffer = (ALdatabuffer*)ALTHUNK_LOOKUPENTRY(buffer);
-
             switch(eParam)
             {
             case AL_SIZE:
-                *plValue = pBuffer->size;
+                *plValue = (ALint)pBuffer->size;
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(pContext, AL_INVALID_ENUM);
                 break;
             }
         }
         else
-            alSetError(AL_INVALID_NAME);
+            alSetError(pContext, AL_INVALID_NAME);
     }
     else
-        alSetError(AL_INVALID_VALUE);
+        alSetError(pContext, AL_INVALID_VALUE);
 
     ProcessContext(pContext);
 }
 
-ALvoid ALAPIENTRY alGetDatabufferivEXT(ALuint buffer, ALenum eParam, ALint* plValues)
+AL_API ALvoid AL_APIENTRY alGetDatabufferivEXT(ALuint buffer, ALenum eParam, ALint* plValues)
 {
     ALCcontext    *pContext;
+    ALCdevice     *Device;
 
     pContext = GetContextSuspended();
     if(!pContext) return;
 
     if(plValues)
     {
-        if(alIsDatabufferEXT(buffer) && buffer != 0)
+        Device = pContext->Device;
+        if(VerifyDatabuffer(Device->DatabufferList, buffer) != NULL)
         {
             switch (eParam)
             {
@@ -506,59 +538,61 @@ ALvoid ALAPIENTRY alGetDatabufferivEXT(ALuint buffer, ALenum eParam, ALint* plVa
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(pContext, AL_INVALID_ENUM);
                 break;
             }
         }
         else
-            alSetError(AL_INVALID_NAME);
+            alSetError(pContext, AL_INVALID_NAME);
     }
     else
-        alSetError(AL_INVALID_VALUE);
+        alSetError(pContext, AL_INVALID_VALUE);
 
     ProcessContext(pContext);
 }
 
 
-ALvoid ALAPIENTRY alSelectDatabufferEXT(ALenum target, ALuint uiBuffer)
+AL_API ALvoid AL_APIENTRY alSelectDatabufferEXT(ALenum target, ALuint uiBuffer)
 {
     ALCcontext    *pContext;
-    ALdatabuffer  *pBuffer;
+    ALdatabuffer  *pBuffer = NULL;
+    ALCdevice     *Device;
 
     pContext = GetContextSuspended();
     if(!pContext) return;
 
-    if(alIsDatabufferEXT(uiBuffer))
+    Device = pContext->Device;
+    if(uiBuffer == 0 ||
+       (pBuffer=VerifyDatabuffer(Device->DatabufferList, uiBuffer)) != NULL)
     {
-        pBuffer = (ALdatabuffer*)(uiBuffer ? ALTHUNK_LOOKUPENTRY(uiBuffer) : NULL);
         if(target == AL_SAMPLE_SOURCE_EXT)
             pContext->SampleSource = pBuffer;
         else if(target == AL_SAMPLE_SINK_EXT)
             pContext->SampleSink = pBuffer;
         else
-            alSetError(AL_INVALID_VALUE);
+            alSetError(pContext, AL_INVALID_VALUE);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(pContext, AL_INVALID_NAME);
 
     ProcessContext(pContext);
 }
 
 
-ALvoid* ALAPIENTRY alMapDatabufferEXT(ALuint uiBuffer, ALuint start, ALsizei length, ALenum access)
+AL_API ALvoid* AL_APIENTRY alMapDatabufferEXT(ALuint uiBuffer, ALintptrEXT start, ALsizeiptrEXT length, ALenum access)
 {
     ALCcontext    *pContext;
     ALdatabuffer  *pBuffer;
     ALvoid        *ret = NULL;
+    ALCdevice     *Device;
 
     pContext = GetContextSuspended();
     if(!pContext) return NULL;
 
-    if(alIsDatabufferEXT(uiBuffer) && uiBuffer != 0)
+    Device = pContext->Device;
+    if((pBuffer=VerifyDatabuffer(Device->DatabufferList, uiBuffer)) != NULL)
     {
-        pBuffer = (ALdatabuffer*)ALTHUNK_LOOKUPENTRY(uiBuffer);
-
-        if(length >= 0 && start+length <= pBuffer->size)
+        if(start >= 0 && length >= 0 && start+length <= pBuffer->size)
         {
             if(access == AL_READ_ONLY_EXT || access == AL_WRITE_ONLY_EXT ||
                access == AL_READ_WRITE_EXT)
@@ -569,41 +603,41 @@ ALvoid* ALAPIENTRY alMapDatabufferEXT(ALuint uiBuffer, ALuint start, ALsizei len
                     pBuffer->state = MAPPED;
                 }
                 else
-                    alSetError(AL_INVALID_OPERATION);
+                    alSetError(pContext, AL_INVALID_OPERATION);
             }
             else
-                alSetError(AL_INVALID_ENUM);
+                alSetError(pContext, AL_INVALID_ENUM);
         }
         else
-            alSetError(AL_INVALID_VALUE);
+            alSetError(pContext, AL_INVALID_VALUE);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(pContext, AL_INVALID_NAME);
 
     ProcessContext(pContext);
 
     return ret;
 }
 
-ALvoid ALAPIENTRY alUnmapDatabufferEXT(ALuint uiBuffer)
+AL_API ALvoid AL_APIENTRY alUnmapDatabufferEXT(ALuint uiBuffer)
 {
     ALCcontext    *pContext;
     ALdatabuffer  *pBuffer;
+    ALCdevice     *Device;
 
     pContext = GetContextSuspended();
     if(!pContext) return;
 
-    if(alIsDatabufferEXT(uiBuffer) && uiBuffer != 0)
+    Device = pContext->Device;
+    if((pBuffer=VerifyDatabuffer(Device->DatabufferList, uiBuffer)) != NULL)
     {
-        pBuffer = (ALdatabuffer*)ALTHUNK_LOOKUPENTRY(uiBuffer);
-
         if(pBuffer->state == MAPPED)
             pBuffer->state = UNMAPPED;
         else
-            alSetError(AL_INVALID_OPERATION);
+            alSetError(pContext, AL_INVALID_OPERATION);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(pContext, AL_INVALID_NAME);
 
     ProcessContext(pContext);
 }
@@ -616,21 +650,18 @@ ALvoid ALAPIENTRY alUnmapDatabufferEXT(ALuint uiBuffer)
 */
 ALvoid ReleaseALDatabuffers(ALCdevice *device)
 {
-    ALdatabuffer *ALBuffer;
-    ALdatabuffer *ALBufferTemp;
-
-    ALBuffer = device->Databuffers;
-    while(ALBuffer)
+    while(device->DatabufferList)
     {
-        // Release sample data
-        free(ALBuffer->data);
+        ALdatabuffer *temp = device->DatabufferList;
+        device->DatabufferList = temp->next;
+
+        // Release buffer data
+        free(temp->data);
 
         // Release Buffer structure
-        ALBufferTemp = ALBuffer;
-        ALBuffer = ALBuffer->next;
-        memset(ALBufferTemp, 0, sizeof(ALdatabuffer));
-        free(ALBufferTemp);
+        ALTHUNK_REMOVEENTRY(temp->databuffer);
+        memset(temp, 0, sizeof(ALdatabuffer));
+        free(temp);
     }
-    device->Databuffers = NULL;
     device->DatabufferCount = 0;
 }
