@@ -36,11 +36,12 @@ ALboolean DisabledEffects[MAX_EFFECTS];
 
 static void InitEffectParams(ALeffect *effect, ALenum type);
 
+DECL_VERIFIER(Effect, ALeffect, effect)
 
-ALvoid AL_APIENTRY alGenEffects(ALsizei n, ALuint *effects)
+AL_API ALvoid AL_APIENTRY alGenEffects(ALsizei n, ALuint *effects)
 {
     ALCcontext *Context;
-    ALsizei i;
+    ALsizei i=0;
 
     Context = GetContextSuspended();
     if(!Context) return;
@@ -52,19 +53,27 @@ ALvoid AL_APIENTRY alGenEffects(ALsizei n, ALuint *effects)
         // Check that enough memory has been allocted in the 'effects' array for n Effects
         if (!IsBadWritePtr((void*)effects, n * sizeof(ALuint)))
         {
+            ALeffect *end;
             ALeffect **list = &device->EffectList;
             while(*list)
                 list = &(*list)->next;
 
-            i = 0;
+            end = *list;
             while(i < n)
             {
                 *list = calloc(1, sizeof(ALeffect));
                 if(!(*list))
                 {
-                    // We must have run out or memory
-                    alDeleteEffects(i, effects);
-                    alSetError(AL_OUT_OF_MEMORY);
+                    while(end->next)
+                    {
+                        ALeffect *temp = end->next;
+                        end->next = temp->next;
+
+                        ALTHUNK_REMOVEENTRY(temp->effect);
+                        device->EffectCount--;
+                        free(temp);
+                    }
+                    alSetError(Context, AL_OUT_OF_MEMORY);
                     break;
                 }
 
@@ -83,7 +92,7 @@ ALvoid AL_APIENTRY alGenEffects(ALsizei n, ALuint *effects)
     ProcessContext(Context);
 }
 
-ALvoid AL_APIENTRY alDeleteEffects(ALsizei n, ALuint *effects)
+AL_API ALvoid AL_APIENTRY alDeleteEffects(ALsizei n, ALuint *effects)
 {
     ALCcontext *Context;
     ALeffect *ALEffect;
@@ -99,9 +108,12 @@ ALvoid AL_APIENTRY alDeleteEffects(ALsizei n, ALuint *effects)
         // Check that all effects are valid
         for (i = 0; i < n; i++)
         {
-            if (!alIsEffect(effects[i]))
+            if(!effects[i])
+                continue;
+
+            if(!VerifyEffect(device->EffectList, effects[i]))
             {
-                alSetError(AL_INVALID_NAME);
+                alSetError(Context, AL_INVALID_NAME);
                 break;
             }
         }
@@ -112,13 +124,11 @@ ALvoid AL_APIENTRY alDeleteEffects(ALsizei n, ALuint *effects)
             for (i = 0; i < n; i++)
             {
                 // Recheck that the effect is valid, because there could be duplicated names
-                if (effects[i] && alIsEffect(effects[i]))
+                if((ALEffect=VerifyEffect(device->EffectList, effects[i])) != NULL)
                 {
                     ALeffect **list;
 
-                    ALEffect = ((ALeffect*)ALTHUNK_LOOKUPENTRY(effects[i]));
-
-                    // Remove Source from list of Sources
+                    // Remove Effect from list of effects
                     list = &device->EffectList;
                     while(*list && *list != ALEffect)
                          list = &(*list)->next;
@@ -136,39 +146,40 @@ ALvoid AL_APIENTRY alDeleteEffects(ALsizei n, ALuint *effects)
         }
     }
     else
-        alSetError(AL_INVALID_VALUE);
+        alSetError(Context, AL_INVALID_VALUE);
 
     ProcessContext(Context);
 }
 
-ALboolean AL_APIENTRY alIsEffect(ALuint effect)
+AL_API ALboolean AL_APIENTRY alIsEffect(ALuint effect)
 {
     ALCcontext *Context;
-    ALeffect *list;
+    ALboolean  result = AL_TRUE;
 
     Context = GetContextSuspended();
     if(!Context) return AL_FALSE;
 
-    list = Context->Device->EffectList;
-    while(list && list->effect != effect)
-        list = list->next;
+    if(effect)
+        result = (VerifyEffect(Context->Device->EffectList, effect) ?
+                  AL_TRUE : AL_FALSE);
 
     ProcessContext(Context);
 
-    return ((list || !effect) ? AL_TRUE : AL_FALSE);
+    return result;
 }
 
-ALvoid AL_APIENTRY alEffecti(ALuint effect, ALenum param, ALint iValue)
+AL_API ALvoid AL_APIENTRY alEffecti(ALuint effect, ALenum param, ALint iValue)
 {
     ALCcontext *Context;
+    ALCdevice  *Device;
+    ALeffect   *ALEffect;
 
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if (effect && alIsEffect(effect))
+    Device = Context->Device;
+    if((ALEffect=VerifyEffect(Device->EffectList, effect)) != NULL)
     {
-        ALeffect *ALEffect = (ALeffect*)ALTHUNK_LOOKUPENTRY(effect);
-
         if(param == AL_EFFECT_TYPE)
         {
             ALboolean isOk = (iValue == AL_EFFECT_NULL ||
@@ -179,7 +190,7 @@ ALvoid AL_APIENTRY alEffecti(ALuint effect, ALenum param, ALint iValue)
             if(isOk)
                 InitEffectParams(ALEffect, iValue);
             else
-                alSetError(AL_INVALID_VALUE);
+                alSetError(Context, AL_INVALID_VALUE);
         }
         else if(ALEffect->type == AL_EFFECT_EAXREVERB)
         {
@@ -190,11 +201,11 @@ ALvoid AL_APIENTRY alEffecti(ALuint effect, ALenum param, ALint iValue)
                    iValue <= AL_EAXREVERB_MAX_DECAY_HFLIMIT)
                     ALEffect->Reverb.DecayHFLimit = iValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -207,11 +218,11 @@ ALvoid AL_APIENTRY alEffecti(ALuint effect, ALenum param, ALint iValue)
                    iValue <= AL_REVERB_MAX_DECAY_HFLIMIT)
                     ALEffect->Reverb.DecayHFLimit = iValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -220,30 +231,31 @@ ALvoid AL_APIENTRY alEffecti(ALuint effect, ALenum param, ALint iValue)
             switch(param)
             {
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
         else
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(Context, AL_INVALID_NAME);
 
     ProcessContext(Context);
 }
 
-ALvoid AL_APIENTRY alEffectiv(ALuint effect, ALenum param, ALint *piValues)
+AL_API ALvoid AL_APIENTRY alEffectiv(ALuint effect, ALenum param, ALint *piValues)
 {
     ALCcontext *Context;
+    ALCdevice  *Device;
+    ALeffect   *ALEffect;
 
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if (effect && alIsEffect(effect))
+    Device = Context->Device;
+    if((ALEffect=VerifyEffect(Device->EffectList, effect)) != NULL)
     {
-        ALeffect *ALEffect = (ALeffect*)ALTHUNK_LOOKUPENTRY(effect);
-
         if(param == AL_EFFECT_TYPE)
         {
             alEffecti(effect, param, piValues[0]);
@@ -257,7 +269,7 @@ ALvoid AL_APIENTRY alEffectiv(ALuint effect, ALenum param, ALint *piValues)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -270,7 +282,7 @@ ALvoid AL_APIENTRY alEffectiv(ALuint effect, ALenum param, ALint *piValues)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -279,30 +291,31 @@ ALvoid AL_APIENTRY alEffectiv(ALuint effect, ALenum param, ALint *piValues)
             switch(param)
             {
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
         else
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(Context, AL_INVALID_NAME);
 
     ProcessContext(Context);
 }
 
-ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
+AL_API ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
 {
     ALCcontext *Context;
+    ALCdevice  *Device;
+    ALeffect   *ALEffect;
 
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if (effect && alIsEffect(effect))
+    Device = Context->Device;
+    if((ALEffect=VerifyEffect(Device->EffectList, effect)) != NULL)
     {
-        ALeffect *ALEffect = (ALeffect*)ALTHUNK_LOOKUPENTRY(effect);
-
         if(ALEffect->type == AL_EFFECT_EAXREVERB)
         {
             switch(param)
@@ -312,7 +325,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_DENSITY)
                     ALEffect->Reverb.Density = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_DIFFUSION:
@@ -320,7 +333,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_DIFFUSION)
                     ALEffect->Reverb.Diffusion = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_GAIN:
@@ -328,7 +341,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_GAIN)
                     ALEffect->Reverb.Gain = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_GAINHF:
@@ -336,7 +349,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_GAIN)
                     ALEffect->Reverb.GainHF = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_GAINLF:
@@ -344,7 +357,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_GAINLF)
                     ALEffect->Reverb.GainLF = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_DECAY_TIME:
@@ -352,7 +365,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_DECAY_TIME)
                     ALEffect->Reverb.DecayTime = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_DECAY_HFRATIO:
@@ -360,7 +373,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_DECAY_HFRATIO)
                     ALEffect->Reverb.DecayHFRatio = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_DECAY_LFRATIO:
@@ -368,7 +381,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_DECAY_LFRATIO)
                     ALEffect->Reverb.DecayLFRatio = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_REFLECTIONS_GAIN:
@@ -376,7 +389,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_REFLECTIONS_GAIN)
                     ALEffect->Reverb.ReflectionsGain = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_REFLECTIONS_DELAY:
@@ -384,7 +397,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_REFLECTIONS_DELAY)
                     ALEffect->Reverb.ReflectionsDelay = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_LATE_REVERB_GAIN:
@@ -392,7 +405,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_LATE_REVERB_GAIN)
                     ALEffect->Reverb.LateReverbGain = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_LATE_REVERB_DELAY:
@@ -400,7 +413,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_LATE_REVERB_DELAY)
                     ALEffect->Reverb.LateReverbDelay = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_AIR_ABSORPTION_GAINHF:
@@ -408,7 +421,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_AIR_ABSORPTION_GAINHF)
                     ALEffect->Reverb.AirAbsorptionGainHF = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_ECHO_TIME:
@@ -416,7 +429,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_ECHO_TIME)
                     ALEffect->Reverb.EchoTime = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_ECHO_DEPTH:
@@ -424,7 +437,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_ECHO_DEPTH)
                     ALEffect->Reverb.EchoDepth = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_MODULATION_TIME:
@@ -432,7 +445,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_MODULATION_TIME)
                     ALEffect->Reverb.ModulationTime = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_MODULATION_DEPTH:
@@ -440,7 +453,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_MODULATION_DEPTH)
                     ALEffect->Reverb.ModulationDepth = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_HFREFERENCE:
@@ -448,7 +461,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_HFREFERENCE)
                     ALEffect->Reverb.HFReference = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_LFREFERENCE:
@@ -456,18 +469,18 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_EAXREVERB_MAX_LFREFERENCE)
                     ALEffect->Reverb.LFReference = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_EAXREVERB_ROOM_ROLLOFF_FACTOR:
                 if(flValue >= 0.0f && flValue <= 10.0f)
                     ALEffect->Reverb.RoomRolloffFactor = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -480,7 +493,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_REVERB_MAX_DENSITY)
                     ALEffect->Reverb.Density = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_REVERB_DIFFUSION:
@@ -488,7 +501,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_REVERB_MAX_DIFFUSION)
                     ALEffect->Reverb.Diffusion = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_REVERB_GAIN:
@@ -496,7 +509,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_REVERB_MAX_GAIN)
                     ALEffect->Reverb.Gain = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_REVERB_GAINHF:
@@ -504,7 +517,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_REVERB_MAX_GAINHF)
                     ALEffect->Reverb.GainHF = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_REVERB_DECAY_TIME:
@@ -512,7 +525,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_REVERB_MAX_DECAY_TIME)
                     ALEffect->Reverb.DecayTime = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_REVERB_DECAY_HFRATIO:
@@ -520,7 +533,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_REVERB_MAX_DECAY_HFRATIO)
                     ALEffect->Reverb.DecayHFRatio = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_REVERB_REFLECTIONS_GAIN:
@@ -528,7 +541,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_REVERB_MAX_REFLECTIONS_GAIN)
                     ALEffect->Reverb.ReflectionsGain = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_REVERB_REFLECTIONS_DELAY:
@@ -536,7 +549,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_REVERB_MAX_REFLECTIONS_DELAY)
                     ALEffect->Reverb.ReflectionsDelay = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_REVERB_LATE_REVERB_GAIN:
@@ -544,7 +557,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_REVERB_MAX_LATE_REVERB_GAIN)
                     ALEffect->Reverb.LateReverbGain = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_REVERB_LATE_REVERB_DELAY:
@@ -552,7 +565,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_REVERB_MAX_LATE_REVERB_DELAY)
                     ALEffect->Reverb.LateReverbDelay = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_REVERB_AIR_ABSORPTION_GAINHF:
@@ -560,7 +573,7 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_REVERB_MAX_AIR_ABSORPTION_GAINHF)
                     ALEffect->Reverb.AirAbsorptionGainHF = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_REVERB_ROOM_ROLLOFF_FACTOR:
@@ -568,11 +581,11 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                    flValue <= AL_REVERB_MAX_ROOM_ROLLOFF_FACTOR)
                     ALEffect->Reverb.RoomRolloffFactor = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -584,62 +597,63 @@ ALvoid AL_APIENTRY alEffectf(ALuint effect, ALenum param, ALfloat flValue)
                 if(flValue >= AL_ECHO_MIN_DELAY && flValue <= AL_ECHO_MAX_DELAY)
                     ALEffect->Echo.Delay = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_ECHO_LRDELAY:
                 if(flValue >= AL_ECHO_MIN_LRDELAY && flValue <= AL_ECHO_MAX_LRDELAY)
                     ALEffect->Echo.LRDelay = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_ECHO_DAMPING:
                 if(flValue >= AL_ECHO_MIN_DAMPING && flValue <= AL_ECHO_MAX_DAMPING)
                     ALEffect->Echo.Damping = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_ECHO_FEEDBACK:
                 if(flValue >= AL_ECHO_MIN_FEEDBACK && flValue <= AL_ECHO_MAX_FEEDBACK)
                     ALEffect->Echo.Feedback = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             case AL_ECHO_SPREAD:
                 if(flValue >= AL_ECHO_MIN_SPREAD && flValue <= AL_ECHO_MAX_SPREAD)
                     ALEffect->Echo.Spread = flValue;
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
         else
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(Context, AL_INVALID_NAME);
 
     ProcessContext(Context);
 }
 
-ALvoid AL_APIENTRY alEffectfv(ALuint effect, ALenum param, ALfloat *pflValues)
+AL_API ALvoid AL_APIENTRY alEffectfv(ALuint effect, ALenum param, ALfloat *pflValues)
 {
     ALCcontext *Context;
+    ALCdevice  *Device;
+    ALeffect   *ALEffect;
 
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if (effect && alIsEffect(effect))
+    Device = Context->Device;
+    if((ALEffect=VerifyEffect(Device->EffectList, effect)) != NULL)
     {
-        ALeffect *ALEffect = (ALeffect*)ALTHUNK_LOOKUPENTRY(effect);
-
         if(ALEffect->type == AL_EFFECT_EAXREVERB)
         {
             switch(param)
@@ -675,7 +689,7 @@ ALvoid AL_APIENTRY alEffectfv(ALuint effect, ALenum param, ALfloat *pflValues)
                     ALEffect->Reverb.ReflectionsPan[2] = pflValues[2];
                 }
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
             case AL_EAXREVERB_LATE_REVERB_PAN:
                 if(!isnan(pflValues[0]) && !isnan(pflValues[1]) && !isnan(pflValues[2]))
@@ -685,11 +699,11 @@ ALvoid AL_APIENTRY alEffectfv(ALuint effect, ALenum param, ALfloat *pflValues)
                     ALEffect->Reverb.LateReverbPan[2] = pflValues[2];
                 }
                 else
-                    alSetError(AL_INVALID_VALUE);
+                    alSetError(Context, AL_INVALID_VALUE);
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -713,7 +727,7 @@ ALvoid AL_APIENTRY alEffectfv(ALuint effect, ALenum param, ALfloat *pflValues)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -730,30 +744,31 @@ ALvoid AL_APIENTRY alEffectfv(ALuint effect, ALenum param, ALfloat *pflValues)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
         else
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(Context, AL_INVALID_NAME);
 
     ProcessContext(Context);
 }
 
-ALvoid AL_APIENTRY alGetEffecti(ALuint effect, ALenum param, ALint *piValue)
+AL_API ALvoid AL_APIENTRY alGetEffecti(ALuint effect, ALenum param, ALint *piValue)
 {
     ALCcontext *Context;
+    ALCdevice  *Device;
+    ALeffect   *ALEffect;
 
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if (effect && alIsEffect(effect))
+    Device = Context->Device;
+    if((ALEffect=VerifyEffect(Device->EffectList, effect)) != NULL)
     {
-        ALeffect *ALEffect = (ALeffect*)ALTHUNK_LOOKUPENTRY(effect);
-
         if(param == AL_EFFECT_TYPE)
         {
             *piValue = ALEffect->type;
@@ -767,7 +782,7 @@ ALvoid AL_APIENTRY alGetEffecti(ALuint effect, ALenum param, ALint *piValue)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -780,7 +795,7 @@ ALvoid AL_APIENTRY alGetEffecti(ALuint effect, ALenum param, ALint *piValue)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -789,30 +804,31 @@ ALvoid AL_APIENTRY alGetEffecti(ALuint effect, ALenum param, ALint *piValue)
             switch(param)
             {
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
         else
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(Context, AL_INVALID_NAME);
 
     ProcessContext(Context);
 }
 
-ALvoid AL_APIENTRY alGetEffectiv(ALuint effect, ALenum param, ALint *piValues)
+AL_API ALvoid AL_APIENTRY alGetEffectiv(ALuint effect, ALenum param, ALint *piValues)
 {
     ALCcontext *Context;
+    ALCdevice  *Device;
+    ALeffect   *ALEffect;
 
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if (effect && alIsEffect(effect))
+    Device = Context->Device;
+    if((ALEffect=VerifyEffect(Device->EffectList, effect)) != NULL)
     {
-        ALeffect *ALEffect = (ALeffect*)ALTHUNK_LOOKUPENTRY(effect);
-
         if(param == AL_EFFECT_TYPE)
         {
             alGetEffecti(effect, param, piValues);
@@ -826,7 +842,7 @@ ALvoid AL_APIENTRY alGetEffectiv(ALuint effect, ALenum param, ALint *piValues)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -839,7 +855,7 @@ ALvoid AL_APIENTRY alGetEffectiv(ALuint effect, ALenum param, ALint *piValues)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -848,30 +864,31 @@ ALvoid AL_APIENTRY alGetEffectiv(ALuint effect, ALenum param, ALint *piValues)
             switch(param)
             {
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
         else
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(Context, AL_INVALID_NAME);
 
     ProcessContext(Context);
 }
 
-ALvoid AL_APIENTRY alGetEffectf(ALuint effect, ALenum param, ALfloat *pflValue)
+AL_API ALvoid AL_APIENTRY alGetEffectf(ALuint effect, ALenum param, ALfloat *pflValue)
 {
     ALCcontext *Context;
+    ALCdevice  *Device;
+    ALeffect   *ALEffect;
 
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if (effect && alIsEffect(effect))
+    Device = Context->Device;
+    if((ALEffect=VerifyEffect(Device->EffectList, effect)) != NULL)
     {
-        ALeffect *ALEffect = (ALeffect*)ALTHUNK_LOOKUPENTRY(effect);
-
         if(ALEffect->type == AL_EFFECT_EAXREVERB)
         {
             switch(param)
@@ -957,7 +974,7 @@ ALvoid AL_APIENTRY alGetEffectf(ALuint effect, ALenum param, ALfloat *pflValue)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -1014,7 +1031,7 @@ ALvoid AL_APIENTRY alGetEffectf(ALuint effect, ALenum param, ALfloat *pflValue)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -1043,30 +1060,31 @@ ALvoid AL_APIENTRY alGetEffectf(ALuint effect, ALenum param, ALfloat *pflValue)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
         else
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(Context, AL_INVALID_NAME);
 
     ProcessContext(Context);
 }
 
-ALvoid AL_APIENTRY alGetEffectfv(ALuint effect, ALenum param, ALfloat *pflValues)
+AL_API ALvoid AL_APIENTRY alGetEffectfv(ALuint effect, ALenum param, ALfloat *pflValues)
 {
     ALCcontext *Context;
+    ALCdevice  *Device;
+    ALeffect   *ALEffect;
 
     Context = GetContextSuspended();
     if(!Context) return;
 
-    if (effect && alIsEffect(effect))
+    Device = Context->Device;
+    if((ALEffect=VerifyEffect(Device->EffectList, effect)) != NULL)
     {
-        ALeffect *ALEffect = (ALeffect*)ALTHUNK_LOOKUPENTRY(effect);
-
         if(ALEffect->type == AL_EFFECT_EAXREVERB)
         {
             switch(param)
@@ -1106,7 +1124,7 @@ ALvoid AL_APIENTRY alGetEffectfv(ALuint effect, ALenum param, ALfloat *pflValues
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -1130,7 +1148,7 @@ ALvoid AL_APIENTRY alGetEffectfv(ALuint effect, ALenum param, ALfloat *pflValues
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
@@ -1147,15 +1165,15 @@ ALvoid AL_APIENTRY alGetEffectfv(ALuint effect, ALenum param, ALfloat *pflValues
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
             }
         }
         else
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
     }
     else
-        alSetError(AL_INVALID_NAME);
+        alSetError(Context, AL_INVALID_NAME);
 
     ProcessContext(Context);
 }
@@ -1163,17 +1181,16 @@ ALvoid AL_APIENTRY alGetEffectfv(ALuint effect, ALenum param, ALfloat *pflValues
 
 ALvoid ReleaseALEffects(ALCdevice *device)
 {
-    ALeffect *list = device->EffectList;
-    while(list)
+    while(device->EffectList)
     {
-        ALeffect *temp = list;
-        list = list->next;
+        ALeffect *temp = device->EffectList;
+        device->EffectList = temp->next;
 
         // Release effect structure
+        ALTHUNK_REMOVEENTRY(temp->effect);
         memset(temp, 0, sizeof(ALeffect));
         free(temp);
     }
-    device->EffectList = NULL;
     device->EffectCount = 0;
 }
 
