@@ -41,7 +41,7 @@ static const ALchar alErrInvalidValue[] = "Invalid Value";
 static const ALchar alErrInvalidOp[] = "Invalid Operation";
 static const ALchar alErrOutOfMemory[] = "Out of Memory";
 
-ALAPI ALvoid ALAPIENTRY alEnable(ALenum capability)
+AL_API ALvoid AL_APIENTRY alEnable(ALenum capability)
 {
     ALCcontext *Context;
     ALboolean  updateSources = AL_FALSE;
@@ -57,13 +57,13 @@ ALAPI ALvoid ALAPIENTRY alEnable(ALenum capability)
             break;
 
         default:
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
             break;
     }
 
     if(updateSources)
     {
-        ALsource *source = Context->Source;
+        ALsource *source = Context->SourceList;
         while(source)
         {
             source->NeedsUpdate = AL_TRUE;
@@ -74,7 +74,7 @@ ALAPI ALvoid ALAPIENTRY alEnable(ALenum capability)
     ProcessContext(Context);
 }
 
-ALAPI ALvoid ALAPIENTRY alDisable(ALenum capability)
+AL_API ALvoid AL_APIENTRY alDisable(ALenum capability)
 {
     ALCcontext *Context;
     ALboolean  updateSources = AL_FALSE;
@@ -90,13 +90,13 @@ ALAPI ALvoid ALAPIENTRY alDisable(ALenum capability)
             break;
 
         default:
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
             break;
     }
 
     if(updateSources)
     {
-        ALsource *source = Context->Source;
+        ALsource *source = Context->SourceList;
         while(source)
         {
             source->NeedsUpdate = AL_TRUE;
@@ -107,7 +107,7 @@ ALAPI ALvoid ALAPIENTRY alDisable(ALenum capability)
     ProcessContext(Context);
 }
 
-ALAPI ALboolean ALAPIENTRY alIsEnabled(ALenum capability)
+AL_API ALboolean AL_APIENTRY alIsEnabled(ALenum capability)
 {
     ALCcontext *Context;
     ALboolean value=AL_FALSE;
@@ -122,7 +122,7 @@ ALAPI ALboolean ALAPIENTRY alIsEnabled(ALenum capability)
             break;
 
         default:
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
             break;
     }
 
@@ -131,7 +131,7 @@ ALAPI ALboolean ALAPIENTRY alIsEnabled(ALenum capability)
     return value;
 }
 
-ALAPI ALboolean ALAPIENTRY alGetBoolean(ALenum pname)
+AL_API ALboolean AL_APIENTRY alGetBoolean(ALenum pname)
 {
     ALCcontext *Context;
     ALboolean value=AL_FALSE;
@@ -162,7 +162,7 @@ ALAPI ALboolean ALAPIENTRY alGetBoolean(ALenum pname)
             break;
 
         default:
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
             break;
     }
 
@@ -171,7 +171,7 @@ ALAPI ALboolean ALAPIENTRY alGetBoolean(ALenum pname)
     return value;
 }
 
-ALAPI ALdouble ALAPIENTRY alGetDouble(ALenum pname)
+AL_API ALdouble AL_APIENTRY alGetDouble(ALenum pname)
 {
     ALCcontext *Context;
     ALdouble value = 0.0;
@@ -198,7 +198,7 @@ ALAPI ALdouble ALAPIENTRY alGetDouble(ALenum pname)
             break;
 
         default:
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
             break;
     }
 
@@ -207,7 +207,7 @@ ALAPI ALdouble ALAPIENTRY alGetDouble(ALenum pname)
     return value;
 }
 
-ALAPI ALfloat ALAPIENTRY alGetFloat(ALenum pname)
+AL_API ALfloat AL_APIENTRY alGetFloat(ALenum pname)
 {
     ALCcontext *Context;
     ALfloat value = 0.0f;
@@ -234,7 +234,7 @@ ALAPI ALfloat ALAPIENTRY alGetFloat(ALenum pname)
             break;
 
         default:
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
             break;
     }
 
@@ -243,7 +243,7 @@ ALAPI ALfloat ALAPIENTRY alGetFloat(ALenum pname)
     return value;
 }
 
-ALAPI ALint ALAPIENTRY alGetInteger(ALenum pname)
+AL_API ALint AL_APIENTRY alGetInteger(ALenum pname)
 {
     ALCcontext *Context;
     ALint value = 0;
@@ -284,7 +284,7 @@ ALAPI ALint ALAPIENTRY alGetInteger(ALenum pname)
             break;
 
         default:
-            alSetError(AL_INVALID_ENUM);
+            alSetError(Context, AL_INVALID_ENUM);
             break;
     }
 
@@ -293,7 +293,7 @@ ALAPI ALint ALAPIENTRY alGetInteger(ALenum pname)
     return value;
 }
 
-ALAPI ALvoid ALAPIENTRY alGetBooleanv(ALenum pname,ALboolean *data)
+AL_API ALvoid AL_APIENTRY alGetBooleanv(ALenum pname,ALboolean *data)
 {
     ALCcontext *Context;
 
@@ -321,20 +321,20 @@ ALAPI ALvoid ALAPIENTRY alGetBooleanv(ALenum pname,ALboolean *data)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
         }
     }
     else
     {
         // data is a NULL pointer
-        alSetError(AL_INVALID_VALUE);
+        alSetError(Context, AL_INVALID_VALUE);
     }
 
     ProcessContext(Context);
 }
 
-ALAPI ALvoid ALAPIENTRY alGetDoublev(ALenum pname,ALdouble *data)
+AL_API ALvoid AL_APIENTRY alGetDoublev(ALenum pname,ALdouble *data)
 {
     ALCcontext *Context;
 
@@ -362,20 +362,20 @@ ALAPI ALvoid ALAPIENTRY alGetDoublev(ALenum pname,ALdouble *data)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
         }
     }
     else
     {
         // data is a NULL pointer
-        alSetError(AL_INVALID_VALUE);
+        alSetError(Context, AL_INVALID_VALUE);
     }
 
     ProcessContext(Context);
 }
 
-ALAPI ALvoid ALAPIENTRY alGetFloatv(ALenum pname,ALfloat *data)
+AL_API ALvoid AL_APIENTRY alGetFloatv(ALenum pname,ALfloat *data)
 {
     ALCcontext *Context;
 
@@ -403,20 +403,20 @@ ALAPI ALvoid ALAPIENTRY alGetFloatv(ALenum pname,ALfloat *data)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
         }
     }
     else
     {
         // data is a NULL pointer
-        alSetError(AL_INVALID_VALUE);
+        alSetError(Context, AL_INVALID_VALUE);
     }
 
     ProcessContext(Context);
 }
 
-ALAPI ALvoid ALAPIENTRY alGetIntegerv(ALenum pname,ALint *data)
+AL_API ALvoid AL_APIENTRY alGetIntegerv(ALenum pname,ALint *data)
 {
     ALCcontext *Context;
 
@@ -458,20 +458,20 @@ ALAPI ALvoid ALAPIENTRY alGetIntegerv(ALenum pname,ALint *data)
                 break;
 
             default:
-                alSetError(AL_INVALID_ENUM);
+                alSetError(Context, AL_INVALID_ENUM);
                 break;
         }
     }
     else
     {
         // data is a NULL pointer
-        alSetError(AL_INVALID_VALUE);
+        alSetError(Context, AL_INVALID_VALUE);
     }
 
     ProcessContext(Context);
 }
 
-ALAPI const ALchar* ALAPIENTRY alGetString(ALenum pname)
+AL_API const ALchar* AL_APIENTRY alGetString(ALenum pname)
 {
     const ALchar *value;
     ALCcontext *pContext;
@@ -523,7 +523,7 @@ ALAPI const ALchar* ALAPIENTRY alGetString(ALenum pname)
 
         default:
             value=NULL;
-            alSetError(AL_INVALID_ENUM);
+            alSetError(pContext, AL_INVALID_ENUM);
             break;
     }
 
@@ -532,7 +532,7 @@ ALAPI const ALchar* ALAPIENTRY alGetString(ALenum pname)
     return value;
 }
 
-ALAPI ALvoid ALAPIENTRY alDopplerFactor(ALfloat value)
+AL_API ALvoid AL_APIENTRY alDopplerFactor(ALfloat value)
 {
     ALCcontext *Context;
     ALboolean updateSources = AL_FALSE;
@@ -546,13 +546,13 @@ ALAPI ALvoid ALAPIENTRY alDopplerFactor(ALfloat value)
         updateSources = AL_TRUE;
     }
     else
-        alSetError(AL_INVALID_VALUE);
+        alSetError(Context, AL_INVALID_VALUE);
 
     // Force updating the sources for these parameters, since even head-
     // relative sources are affected
     if(updateSources)
     {
-        ALsource *source = Context->Source;
+        ALsource *source = Context->SourceList;
         while(source)
         {
             source->NeedsUpdate = AL_TRUE;
@@ -563,7 +563,7 @@ ALAPI ALvoid ALAPIENTRY alDopplerFactor(ALfloat value)
     ProcessContext(Context);
 }
 
-ALAPI ALvoid ALAPIENTRY alDopplerVelocity(ALfloat value)
+AL_API ALvoid AL_APIENTRY alDopplerVelocity(ALfloat value)
 {
     ALCcontext *Context;
     ALboolean updateSources = AL_FALSE;
@@ -577,11 +577,11 @@ ALAPI ALvoid ALAPIENTRY alDopplerVelocity(ALfloat value)
         updateSources = AL_TRUE;
     }
     else
-        alSetError(AL_INVALID_VALUE);
+        alSetError(Context, AL_INVALID_VALUE);
 
     if(updateSources)
     {
-        ALsource *source = Context->Source;
+        ALsource *source = Context->SourceList;
         while(source)
         {
             source->NeedsUpdate = AL_TRUE;
@@ -592,7 +592,7 @@ ALAPI ALvoid ALAPIENTRY alDopplerVelocity(ALfloat value)
     ProcessContext(Context);
 }
 
-ALAPI ALvoid ALAPIENTRY alSpeedOfSound(ALfloat flSpeedOfSound)
+AL_API ALvoid AL_APIENTRY alSpeedOfSound(ALfloat flSpeedOfSound)
 {
     ALCcontext *pContext;
     ALboolean updateSources = AL_FALSE;
@@ -606,11 +606,11 @@ ALAPI ALvoid ALAPIENTRY alSpeedOfSound(ALfloat flSpeedOfSound)
         updateSources = AL_TRUE;
     }
     else
-        alSetError(AL_INVALID_VALUE);
+        alSetError(pContext, AL_INVALID_VALUE);
 
     if(updateSources)
     {
-        ALsource *source = pContext->Source;
+        ALsource *source = pContext->SourceList;
         while(source)
         {
             source->NeedsUpdate = AL_TRUE;
@@ -621,7 +621,7 @@ ALAPI ALvoid ALAPIENTRY alSpeedOfSound(ALfloat flSpeedOfSound)
     ProcessContext(pContext);
 }
 
-ALAPI ALvoid ALAPIENTRY alDistanceModel(ALenum value)
+AL_API ALvoid AL_APIENTRY alDistanceModel(ALenum value)
 {
     ALCcontext *Context;
     ALboolean updateSources = AL_FALSE;
@@ -643,13 +643,13 @@ ALAPI ALvoid ALAPIENTRY alDistanceModel(ALenum value)
             break;
 
         default:
-            alSetError(AL_INVALID_VALUE);
+            alSetError(Context, AL_INVALID_VALUE);
             break;
     }
 
     if(updateSources)
     {
-        ALsource *source = Context->Source;
+        ALsource *source = Context->SourceList;
         while(source)
         {
             source->NeedsUpdate = AL_TRUE;
